@@ -1,16 +1,13 @@
 sem.missing.paths = function(
  
   modelList, data, conditional = FALSE, corr.errors = NULL, add.vars = NULL, grouping.vars = NULL, 
-  top.level.vars = NULL, adjust.p = FALSE, basis.set = NULL, model.control = NULL, .progressBar = TRUE
+  adjust.p = FALSE, basis.set = NULL, model.control = NULL, .progressBar = TRUE
   
   ) {
   
   # Get basis set
   if(is.null(basis.set)) basis.set = suppressWarnings(sem.basis.set(modelList, corr.errors, add.vars))
 
-  # Filter exogenous variables
-  basis.set = filter.exogenous(modelList, basis.set, corr.errors, add.vars) 
-
   # Add progress bar
   if(.progressBar == T & length(basis.set) > 0) 
     
@@ -20,8 +17,14 @@ sem.missing.paths = function(
   if(length(basis.set) > 0) pvalues.df = do.call(rbind, lapply(1:length(basis.set), function(i) {
     
     # Get basis model from which to build the d-sep test
-    basis.mod = modelList[[match(basis.set[[i]][2], sapply(modelList, function(j) strsplit(paste(formula(j)), "~")[[2]]))]]
+    basis.mod = modelList[[which(sapply(modelList, function(j) {
+      
+      if(any(class(j) == "pgls")) j = j$formula
+      
+      rownames(attr(terms(j), "factors"))[1] == basis.set[[i]][2]
     
+    } ) ) ]]
+      
     # Get fixed formula
     rhs = if(length(basis.set[[i]]) <= 2) paste(basis.set[[i]][1]) else
       
@@ -31,40 +34,59 @@ sem.missing.paths = function(
     random.formula = get.random.formula(basis.mod, rhs, modelList)
     
     # Aggregate at the level of the grouping variable
-    if(!is.null(grouping.vars) & any(top.level.vars %in% gsub(".*\\((.*)\\)", "\\1", unlist(as.character(formula(basis.mod)[2])))))
+    if(!is.null(grouping.vars)) {
       
-      data = suppressWarnings(aggregate(data, by = lapply(grouping.vars, function(i) data[ ,i]), mean, na.rm = T))
+      # Test to see if response is identical for levels of grouping factor
+      response = as.character(formula(basis.mod)[2])
+      
+      response.test = by(data, lapply(grouping.vars, function(j) data[ ,j]), function(x) length(unique(x[, response])) == 1)
+       
+      response.test = na.omit(vapply(response.test, unlist, unlist(response.test[[1]])))
+       
+      # If so, aggregate by grouping.vars
+      if(all(response.test == TRUE)) {
+        
+        # Get named list
+        groups = lapply(grouping.vars, function(j) data[ ,j])
+        
+        names(groups) = grouping.vars
+        
+        # Aggregate and replace data
+        data = suppressWarnings(
+          
+          aggregate(data, by = groups, mean, na.rm = T)
+          
+          )
+        
+        # Remove duplicated colnames (keep first instance, from above)
+        data = data[, !duplicated(colnames(data))]
+        
+      }
+    
+    }
     
     # Get model controls
     control = get.model.control(basis.mod, model.control)
     
     # Update basis model with new formula and random structure based on d-sep
-    basis.mod.new = suppressWarnings(if(is.null(random.formula)) 
+    basis.mod.new = suppressWarnings(
       
-      update(basis.mod, formula(paste(basis.set[[i]][2], " ~ ", rhs)), data = data) else
+      if(is.null(random.formula) | class(basis.mod) == "glmmadmb") 
+      
+        update(basis.mod, formula(paste(basis.set[[i]][2], " ~ ", rhs)), data = data) else
         
-        if(any(class(basis.mod) %in% c("lme", "glmmPQL"))) 
+          if(any(class(basis.mod) %in% c("lme", "glmmPQL"))) 
           
-          update(basis.mod, fixed = formula(paste(basis.set[[i]][2], " ~ ", rhs)), random = random.formula, control = control, data = data) else
+            update(basis.mod, fixed = formula(paste(basis.set[[i]][2], " ~ ", rhs)), random = random.formula, control = control, data = data) else
             
-            update(basis.mod, formula = formula(paste(basis.set[[i]][2], " ~ ", rhs, " + ", random.formula, sep = "")), control = control, data = data) 
+              update(basis.mod, formula = formula(paste(basis.set[[i]][2], " ~ ", rhs, " + ", random.formula, sep = "")), control = control, data = data) 
       
     )
-    
-    # Stop if lmerTest does not return p-values
-    if("lmerMod" %in% class(basis.mod.new)) {
-      
-      basis.mod.new = as(basis.mod.new, "merModLmerTest") 
-      
-      x = try(suppressMessages(summary(basis.mod.new)$coefficients[1,5]), silent = T)
-      
-      if(class(x) == "try-error") stop("lmerTest did not return p-values. Consider using functions in the nlme package.")
-      
-    }
-    
+
     # Get row number from coefficient table for d-sep variable
-    if(any(!class(basis.mod.new) %in% "pgls")) {
+    if(any(!class(basis.mod.new) %in% c("pgls"))) {
       
+      # Get row number of d-sep claim
       row.num = which(basis.set[[i]][1] == rownames(attr(terms(basis.mod.new), "factors"))[-1]) + 1 
       
       # Get row number if interaction variables are switched
@@ -95,7 +117,26 @@ sem.missing.paths = function(
       }
     
     # Return new coefficient table
-    ret = if(any(class(basis.mod.new) %in% c("lm", "glm", "negbin", "pgls", "glmerMod", "merModLmerTest")))
+    ret = if(any(class(basis.mod.new) %in% c("lmerMod", "merModLmerTest"))) {
+      
+      coef.table = suppressMessages(summary(basis.mod.new)$coefficients)
+      
+      # Get Kenward-Rogers approximation of denominator degrees of freedom
+      kr.ddf = suppressMessages(get_ddf_Lb(basis.mod.new, fixef(basis.mod.new)))
+      
+      # Compute p-values based on t-distribution and ddf
+      kr.p = 2 * (1 - pt(abs(coef.table[, "t value"]), kr.ddf))[row.num]
+      
+      # Combine with coefficients from regular ouput
+      data.frame(
+        t(coef.table[row.num, 1:2]),
+        kr.ddf,
+        coef.table[row.num, 3],
+        kr.p,
+        row.names = NULL
+      )
+      
+    } else if(any(class(basis.mod.new) %in% c("lm", "glm", "negbin", "pgls", "glmerMod", "glmmadmb")))
       
       as.data.frame(t(unname(summary(basis.mod.new)$coefficients[row.num, ]))) else
       
@@ -106,34 +147,50 @@ sem.missing.paths = function(
       
       ret = cbind(ret[1:2], summary(basis.mod.new)$df[2], ret[3:4]) else
         
-        if(length(ret) != 5)
+        if(length(ret) != 5 & any(class(basis.mod.new) %in% c("glmmadmb"))) 
           
-          ret = cbind(ret[1:2], NA, ret[3:4])
+          ret = cbind(ret[1:2], summary(basis.mod.new)$n, ret[3:4]) else
+            
+            if(length(ret) != 5)
+              
+              ret = cbind(ret[1:2], NA, ret[3:4])
       
     # Rename columns 
-    names(ret) = c("estimate", "std.error", "DF", "crit.value", "p.value")
+    names(ret) = c("estimate", "std.error", "df", "crit.value", "p.value")
     
     # Adjust p-value based on Shipley 2013
     if(adjust.p == TRUE) {
       
-      if(all(class(basis.mod.new) %in% c("lme", "glmmPQL"))) {
+      if(any(class(basis.mod.new) %in% c("lme", "glmmPQL"))) {
         
         t.value = summary(basis.mod.new)$tTable[row.num, 4] 
         
         ret[5] = 2*(1 - pt(abs(t.value), nobs(basis.mod.new) - sum(apply(basis.mod.new$groups, 2, function(x) length(unique(x))))))
         
-      } else if(all(class(basis.mod.new) %in% c("glmerMod", "merModLmerTest"))) {
+      } else if(any(class(basis.mod.new) %in% c("lmerMod", "glmerMod", "merModLmerTest"))) {
         
-        z.value = summary(basis.mod.new)$coefficients[row.num, 4]
+        z.value = coef.table[row.num, "t value"]
         
-        ret[5] = 2*(1 - pt(abs(z.value), nobs(basis.mod.new) - sum(summary(basis.mod.new)$ngrps))) } 
+        ret[5] = 2*(1 - pt(abs(z.value), nobs(basis.mod.new) - sum(summary(basis.mod.new)$ngrps))) 
+        
+        } else if(any(class(basis.mod.new) %in% c("glmmadmb"))) {
+          
+          z.value = summary(basis.mod.new)$coefficients[row.num, 3]
+          
+          ret[5] = 2*(1 - pt(abs(z.value), nobs(basis.mod.new) - sum(summary(basis.mod.new)$npar))) 
+      
+        }
       
     }
   
     if(.progressBar == TRUE) setTxtProgressBar(pb, i)
     
     # Modify rhs if number of characters exceeds 20
-    if(conditional == FALSE) if(nchar(rhs) > 30) rhs = paste(gsub(".\\+.*$", "", rhs), "+ ...")
+    if(conditional == FALSE & nchar(rhs) > 30) {
+      
+      rhs = paste(gsub(".\\+.*$", "", rhs), "+ ...")
+      
+    }
     
     # Bind in d-sep metadata
     data.frame(missing.path = paste(basis.set[[i]][2], " ~ ", rhs, sep = ""), ret)
@@ -143,13 +200,14 @@ sem.missing.paths = function(
     pvalues.df = data.frame(missing.path = NA, estimate = NA, std.error = NA, DF = NA, crit.value = NA, p.value = NA)
   
   # Set degrees of freedom as numeric
-  pvalues.df$DF = as.numeric(pvalues.df$DF)
-  
-  # Output warning if missing.paths contains ...
-  if(any(grepl("\\.\\.\\.", pvalues.df$missing.path))) print("Conditional variables have been omitted from output table for clarity")
+  pvalues.df$df = as.numeric(pvalues.df$df)
   
   if(!is.null(pb)) close(pb)  
   
+  if(any(grepl("...", pvalues.df$missing.path))) 
+    
+    message("Conditional variables have been omitted from output table for clarity (or use argument conditional = T)")
+  
   return(pvalues.df)
   
 }
\ No newline at end of file
